@@ -12,17 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-# NOTE: jinja2 3.1.0 dropped contextfilter in favour of pass_context.
-try:
-    from jinja2 import pass_context
-except ImportError:
-    from jinja2 import contextfilter as pass_context
+import jinja2
 
 from kolla_ansible import exception
 from kolla_ansible.helpers import _call_bool_filter
 
 
-@pass_context
+@jinja2.pass_context
 def service_enabled(context, service):
     """Return whether a service is enabled.
 
@@ -38,7 +34,7 @@ def service_enabled(context, service):
     return _call_bool_filter(context, enabled)
 
 
-@pass_context
+@jinja2.pass_context
 def service_mapped_to_host(context, service):
     """Return whether a service is mapped to this host.
 
@@ -66,7 +62,7 @@ def service_mapped_to_host(context, service):
         service.get("container_name", "<unknown>"))
 
 
-@pass_context
+@jinja2.pass_context
 def service_enabled_and_mapped_to_host(context, service):
     """Return whether a service is enabled and mapped to this host.
 
@@ -78,7 +74,7 @@ def service_enabled_and_mapped_to_host(context, service):
             service_mapped_to_host(context, service))
 
 
-@pass_context
+@jinja2.pass_context
 def select_services_enabled_and_mapped_to_host(context, services):
     """Select services that are enabled and mapped to this host.
 
