@@ -32,6 +32,9 @@ copy_logs() {
     lsblk > ${LOG_DIR}/system_logs/lsblk.txt
     mount > ${LOG_DIR}/system_logs/mount.txt
     env > ${LOG_DIR}/system_logs/env.txt
+    systemctl status > ${LOG_DIR}/system_logs/systemctl_status.txt
+    systemctl list-units --all > ${LOG_DIR}/system_logs/systemctl_units.txt
+    systemctl list-unit-files > ${LOG_DIR}/system_logs/systemctl_unit_files.txt
 
     (set -x
     ip a
@@ -80,8 +83,13 @@ copy_logs() {
     # docker related information
     (docker info && docker images && docker ps -a && docker network ls && docker inspect $(docker ps -aq)) > ${LOG_DIR}/system_logs/docker-info.txt
 
+    # save dbus services
+    dbus-send --system --print-reply --dest=org.freedesktop.DBus /org/freedesktop/DBus org.freedesktop.DBus.ListNames > ${LOG_DIR}/system_logs/dbus-services.txt
+
     # cephadm related logs
     mkdir -p ${LOG_DIR}/ceph
+    sudo cp /etc/ceph/ceph.conf ${LOG_DIR}/ceph
+    sudo cp -Rf /var/run/ceph/* ${LOG_DIR}/ceph
     sudo cephadm shell -- ceph --connect-timeout 5 -s > ${LOG_DIR}/ceph/ceph_s.txt
     sudo cephadm shell -- ceph --connect-timeout 5 osd tree > ${LOG_DIR}/ceph/ceph_osd_tree.txt
 
