@@ -71,6 +71,8 @@ class ModuleArgsTest(base.BaseTestCase):
             cap_add=dict(required=False, type='list', default=list()),
             security_opt=dict(required=False, type='list', default=list()),
             pid_mode=dict(required=False, type='str', choices=['host', '']),
+            cgroupns_mode=dict(required=False, type='str',
+                               choices=['private', 'host']),
             privileged=dict(required=False, type='bool', default=False),
             graceful_timeout=dict(required=False, type='int', default=10),
             remove_on_exit=dict(required=False, type='bool', default=True),
@@ -88,11 +90,13 @@ class ModuleArgsTest(base.BaseTestCase):
             tls_cert=dict(required=False, type='str'),
             tls_key=dict(required=False, type='str'),
             tls_cacert=dict(required=False, type='str'),
+            tmpfs=dict(required=False, type='list'),
             volumes=dict(required=False, type='list'),
             volumes_from=dict(required=False, type='list'),
             dimensions=dict(required=False, type='dict', default=dict()),
             tty=dict(required=False, type='bool', default=False),
             client_timeout=dict(required=False, type='int', default=120),
+            healthcheck=dict(required=False, type='dict'),
             ignore_missing=dict(required=False, type='bool', default=False),
         )
         required_if = [
@@ -135,6 +139,7 @@ FAKE_DATA = {
             'security_opt': None,
             'pid_mode': '',
             'privileged': False,
+            'tmpfs': None,
             'volumes_from': None,
             'restart_policy': 'unless-stopped',
             'restart_retries': 10},
@@ -201,12 +206,13 @@ FAKE_DATA = {
 }
 
 
-@mock.patch("docker.APIClient")
-def get_DockerWorker(mod_param, mock_dclient):
+def get_DockerWorker(mod_param, docker_api_version='1.40'):
     module = mock.MagicMock()
     module.params = mod_param
-    dw = kd.DockerWorker(module)
-    return dw
+    with mock.patch("docker.APIClient") as MockedDockerClientClass:
+        MockedDockerClientClass.return_value._version = docker_api_version
+        dw = kd.DockerWorker(module)
+        return dw
 
 
 class TestMainModule(base.BaseTestCase):
@@ -245,6 +251,15 @@ class TestMainModule(base.BaseTestCase):
                                                       result=False,
                                                       some_key="some_value")
 
+    def test_sets_cgroupns_mode_supported_false(self):
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertFalse(self.dw._cgroupns_mode_supported)
+
+    def test_sets_cgroupns_mode_supported_true(self):
+        self.dw = get_DockerWorker(self.fake_data['params'],
+                                   docker_api_version='1.41')
+        self.assertTrue(self.dw._cgroupns_mode_supported)
+
 
 class TestContainer(base.BaseTestCase):
 
@@ -268,13 +283,14 @@ class TestContainer(base.BaseTestCase):
         self.assertTrue(self.dw.changed)
         self.fake_data['params'].pop('dimensions')
         self.fake_data['params']['host_config']['blkio_weight'] = '10'
-        expected_args = {'command', 'detach', 'environment', 'host_config',
-                         'image', 'labels', 'name', 'tty', 'volumes'}
+        expected_args = {'command', 'detach', 'environment',
+                         'host_config', 'image', 'labels', 'name', 'tty',
+                         'volumes'}
         self.dw.dc.create_container.assert_called_once_with(
             **{k: self.fake_data['params'][k] for k in expected_args})
         self.dw.dc.create_host_config.assert_called_with(
             cap_add=None, network_mode='host', ipc_mode=None,
-            pid_mode=None, volumes_from=None, blkio_weight=10,
+            pid_mode=None, tmpfs=None, volumes_from=None, blkio_weight=10,
             security_opt=None, privileged=None)
 
     def test_create_container_wrong_dimensions(self):
@@ -287,6 +303,39 @@ class TestContainer(base.BaseTestCase):
             failed=True, msg=repr("Unsupported dimensions"),
             unsupported_dimensions=set(['random']))
 
+    def test_create_container_with_healthcheck(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh']}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.dc.create_host_config = mock.MagicMock(
+            return_value=self.fake_data['params']['host_config'])
+        self.dw.create_container()
+        self.assertTrue(self.dw.changed)
+        expected_args = {'command', 'detach', 'environment', 'host_config',
+                         'healthcheck', 'image', 'labels', 'name', 'tty',
+                         'volumes'}
+        self.dw.dc.create_container.assert_called_once_with(
+            **{k: self.fake_data['params'][k] for k in expected_args})
+
+    def test_create_container_with_tmpfs(self):
+        self.fake_data['params']['tmpfs'] = ['/tmp']  # nosec: B108
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.dc.create_host_config = mock.MagicMock(
+            return_value=self.fake_data['params']['host_config'])
+        self.dw.create_container()
+        self.assertTrue(self.dw.changed)
+        self.assertEqual(['/tmp'],  # nosec: B108
+                         self.dw.dc.create_host_config.call_args[1]['tmpfs'])
+
+    def test_create_container_with_tmpfs_empty_string(self):
+        self.fake_data['params']['tmpfs'] = ['']
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.dc.create_host_config = mock.MagicMock(
+            return_value=self.fake_data['params']['host_config'])
+        self.dw.create_container()
+        self.assertTrue(self.dw.changed)
+        self.assertFalse(self.dw.dc.create_host_config.call_args[1]['tmpfs'])
+
     def test_start_container_without_pull(self):
         self.fake_data['params'].update({'auth_username': 'fake_user',
                                          'auth_password': 'fake_psw',
@@ -521,8 +570,8 @@ class TestContainer(base.BaseTestCase):
         State = {'Dead': False,
                  'ExitCode': 0,
                  'Pid': 12475,
-                 'StartedAt': u'2016-06-07T11:22:37.66876269Z',
-                 'Status': u'running'}
+                 'StartedAt': '2016-06-07T11:22:37.66876269Z',
+                 'Status': 'running'}
         self.fake_data['container_inspect'].update({'State': State})
         self.dw = get_DockerWorker({'name': 'my_container',
                                     'action': 'get_container_state'})
@@ -926,7 +975,7 @@ class TestVolume(base.BaseTestCase):
         self.fake_data = copy.deepcopy(FAKE_DATA)
         self.volumes = {
             'Volumes':
-            [{'Driver': u'local',
+            [{'Driver': 'local',
               'Labels': None,
               'Mountpoint': '/var/lib/docker/volumes/nova_compute/_data',
               'Name': 'nova_compute'},
@@ -1037,6 +1086,40 @@ class TestAttrComp(base.BaseTestCase):
         self.dw = get_DockerWorker({'pid_mode': 'host2'})
         self.assertTrue(self.dw.compare_pid_mode(container_info))
 
+    def test_compare_cgroupns_mode_neg(self):
+        container_info = {'HostConfig': dict(CgroupnsMode='host')}
+        self.dw = get_DockerWorker({'cgroupns_mode': 'host'},
+                                   docker_api_version='1.41')
+        self.assertFalse(self.dw.compare_cgroupns_mode(container_info))
+
+    def test_compare_cgroupns_mode_neg_backward_compat(self):
+        container_info = {'HostConfig': dict(CgroupnsMode='')}
+        self.dw = get_DockerWorker({'cgroupns_mode': 'host'},
+                                   docker_api_version='1.41')
+        self.assertFalse(self.dw.compare_cgroupns_mode(container_info))
+
+    def test_compare_cgroupns_mode_ignore(self):
+        container_info = {'HostConfig': dict(CgroupnsMode='private')}
+        self.dw = get_DockerWorker({}, docker_api_version='1.41')
+        self.assertFalse(self.dw.compare_cgroupns_mode(container_info))
+
+    def test_compare_cgroupns_mode_pos(self):
+        container_info = {'HostConfig': dict(CgroupnsMode='private')}
+        self.dw = get_DockerWorker({'cgroupns_mode': 'host'},
+                                   docker_api_version='1.41')
+        self.assertTrue(self.dw.compare_cgroupns_mode(container_info))
+
+    def test_compare_cgroupns_mode_pos_backward_compat(self):
+        container_info = {'HostConfig': dict(CgroupnsMode='')}
+        self.dw = get_DockerWorker({'cgroupns_mode': 'private'},
+                                   docker_api_version='1.41')
+        self.assertTrue(self.dw.compare_cgroupns_mode(container_info))
+
+    def test_compare_cgroupns_mode_unsupported(self):
+        container_info = {'HostConfig': dict()}
+        self.dw = get_DockerWorker({'cgroupns_mode': 'host'})
+        self.assertFalse(self.dw.compare_cgroupns_mode(container_info))
+
     def test_compare_privileged_neg(self):
         container_info = {'HostConfig': dict(Privileged=True)}
         self.dw = get_DockerWorker({'privileged': True})
@@ -1061,6 +1144,36 @@ class TestAttrComp(base.BaseTestCase):
             Labels={'kolla_version': '1.0.1'}))
         self.assertTrue(self.dw.compare_labels(container_info))
 
+    def test_compare_tmpfs_neg(self):
+        container_info = {'HostConfig': dict(Tmpfs=['foo'])}
+        self.dw = get_DockerWorker({'tmpfs': ['foo']})
+
+        self.assertFalse(self.dw.compare_tmpfs(container_info))
+
+    def test_compare_tmpfs_neg_empty_string(self):
+        container_info = {'HostConfig': dict()}
+        self.dw = get_DockerWorker({'tmpfs': ['']})
+
+        self.assertFalse(self.dw.compare_tmpfs(container_info))
+
+    def test_compare_tmpfs_pos_different(self):
+        container_info = {'HostConfig': dict(Tmpfs=['foo'])}
+        self.dw = get_DockerWorker({'tmpfs': ['bar']})
+
+        self.assertTrue(self.dw.compare_tmpfs(container_info))
+
+    def test_compare_tmpfs_pos_empty_new(self):
+        container_info = {'HostConfig': dict(Tmpfs=['foo'])}
+        self.dw = get_DockerWorker({})
+
+        self.assertTrue(self.dw.compare_tmpfs(container_info))
+
+    def test_compare_tmpfs_pos_empty_current(self):
+        container_info = {'HostConfig': dict()}
+        self.dw = get_DockerWorker({'tmpfs': ['bar']})
+
+        self.assertTrue(self.dw.compare_tmpfs(container_info))
+
     def test_compare_volumes_from_neg(self):
         container_info = {'HostConfig': dict(VolumesFrom=['777f7dc92da7'])}
         self.dw = get_DockerWorker({'volumes_from': ['777f7dc92da7']})
@@ -1234,3 +1347,236 @@ class TestAttrComp(base.BaseTestCase):
             'Ulimits': [ulimits_nofile]}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertFalse(self.dw.compare_dimensions(container_info))
+
+    def test_compare_empty_new_healthcheck(self):
+        container_info = dict()
+        container_info['Config'] = {
+            'Healthcheck': {
+                'Test': [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_healthcheck(container_info))
+
+    def test_compare_empty_current_healthcheck(self):
+        self.fake_data['params']['healthcheck'] = {
+            'test': ['CMD-SHELL', '/bin/check.sh'],
+            'interval': 30,
+            'timeout': 30,
+            'start_period': 5,
+            'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_no_test(self):
+        self.fake_data['params']['healthcheck'] = {
+            'interval': 30,
+            'timeout': 30,
+            'start_period': 5,
+            'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {
+            'Healthcheck': {
+                'Test': [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.compare_healthcheck(container_info)
+        self.dw.module.exit_json.assert_called_once_with(
+            failed=True, msg=repr("Missing healthcheck option"),
+            missing_healthcheck=set(['test']))
+
+    def test_compare_healthcheck_pos(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD', '/bin/check']}
+        container_info = dict()
+        container_info['Config'] = {
+            'Healthcheck': {
+                'Test': [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_neg(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'interval': 30,
+             'timeout': 30,
+             'start_period': 5,
+             'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertFalse(self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_time_zero(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'interval': 0,
+             'timeout': 30,
+             'start_period': 5,
+             'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_time_wrong_type(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'timeout': 30,
+             'start_period': 5,
+             'retries': 3}
+        self.fake_data['params']['healthcheck']['interval'] = \
+            {"broken": {"interval": "True"}}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertRaises(TypeError,
+                          lambda: self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_time_wrong_value(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'timeout': 30,
+             'start_period': 5,
+             'retries': 3}
+        self.fake_data['params']['healthcheck']['interval'] = "dog"
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertRaises(ValueError,
+                          lambda: self.dw.compare_healthcheck(container_info))
+
+    def test_compare_healthcheck_opt_missing(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'interval': 30,
+             'timeout': 30,
+             'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.compare_healthcheck(container_info)
+        self.dw.module.exit_json.assert_called_once_with(
+            failed=True, msg=repr("Missing healthcheck option"),
+            missing_healthcheck=set(['start_period']))
+
+    def test_compare_healthcheck_opt_extra(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'interval': 30,
+             'start_period': 5,
+             'extra_option': 1,
+             'timeout': 30,
+             'retries': 3}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.dw.compare_healthcheck(container_info)
+        self.dw.module.exit_json.assert_called_once_with(
+            failed=True, msg=repr("Unsupported healthcheck options"),
+            unsupported_healthcheck=set(['extra_option']))
+
+    def test_compare_healthcheck_value_false(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['CMD-SHELL', '/bin/check.sh'],
+             'interval': 30,
+             'start_period': 5,
+             'extra_option': 1,
+             'timeout': 30,
+             'retries': False}
+        container_info = dict()
+        container_info['Config'] = {
+            "Healthcheck": {
+                "Test": [
+                    "CMD-SHELL",
+                    "/bin/check.sh"],
+                "Interval": 30000000000,
+                "Timeout": 30000000000,
+                "StartPeriod": 5000000000,
+                "Retries": 3}}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_healthcheck(container_info))
+
+    def test_parse_healthcheck_empty(self):
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertIsNone(self.dw.parse_healthcheck(
+                          self.fake_data.get('params', {}).get('healthcheck')))
+
+    def test_parse_healthcheck_test_none(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': 'NONE'}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertIsNone(self.dw.parse_healthcheck(
+                          self.fake_data['params']['healthcheck']))
+
+    def test_parse_healthcheck_test_none_brackets(self):
+        self.fake_data['params']['healthcheck'] = \
+            {'test': ['NONE']}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertIsNone(self.dw.parse_healthcheck(
+                          self.fake_data['params']['healthcheck']))
